@@ -235,7 +235,7 @@ class Chemical:
              Pt: 610.88 Pa
              Pc: 2.2048e+07 Pa
              Vc: 5.6e-05 m^3/mol
-             Hf: -2.4182e+05 J/mol
+             Hf: -2.8582e+05 J/mol
              LHV: 44011 J/mol
              HHV: 0 J/mol
              Hfus: 6010 J/mol
@@ -499,7 +499,8 @@ class Chemical:
                 else:
                     cache[ID] = self = self.copy(ID=ID, CAS=self._CAS)
             setfield = setattr
-            for i,j in data.items(): setfield(self, i, j)
+            for i,j in data.items(): 
+                if j is not None: setfield(self, i, j)
         else:
             self = super().__new__(cls)
             self._ID = ID
@@ -722,7 +723,7 @@ class Chemical:
         return self._formula
     @formula.setter
     def formula(self, formula):
-        # if self._formula: raise AttributeError('cannot set formula')
+        if self._formula: raise AttributeError('cannot set formula')
         self._formula = str(formula)
         if self._Hf is None:
             self.MW = compute_molecular_weight(self.atoms)
@@ -970,7 +971,7 @@ class Chemical:
     
     @property
     def Hf(self):
-        """Heat of formation [J/mol]."""
+        """Heat of formation at reference phase and temperature [J/mol]."""
         return self._Hf
     @Hf.setter
     def Hf(self, Hf):
@@ -1039,9 +1040,9 @@ class Chemical:
     def atoms(self):
         """dict[str: int] Atom-count pairs based on formula."""
         formula = self._formula
-        return get_atoms(formula) if formula else None
+        return get_atoms(formula) if formula else {}
     
-    def get_combustion_reaction(self, chemicals):
+    def get_combustion_reaction(self, chemicals=None):
         """Return a Reaction object defining the combustion of this chemical.
         If no combustion stoichiometry is available, return None."""
         combustion = self._combustion
@@ -1115,16 +1116,16 @@ class Chemical:
         except:
             pass            
         else:
-            # if formula and info.formula and get_atoms(formula) != get_atoms(info.formula):
-            #     raise RuntimeError(
-            #         f'{self.ID} (CAS: {CAS}) formula from database, '
-            #         f'{info.formula}, does not match with user '
-            #         f'specification, {formula}')
-            # if MW and info.MW and MW != info.MW:
-            #     raise RuntimeError(
-            #         f'{self.ID} (CAS: {CAS}) molecular weight from database, '
-            #         f'{info.MW:.2f} g/mol, does not match with user '
-            #         f'specification, {MW:.2f} g/mol')
+            if formula and info.formula and get_atoms(formula) != get_atoms(info.formula):
+                raise RuntimeError(
+                    f'{self.ID} (CAS: {CAS}) formula from database, '
+                    f'{info.formula}, does not match with user '
+                    f'specification, {formula}')
+            if MW and info.MW and MW != info.MW:
+                raise RuntimeError(
+                    f'{self.ID} (CAS: {CAS}) molecular weight from database, '
+                    f'{info.MW:.2f} g/mol, does not match with user '
+                    f'specification, {MW:.2f} g/mol')
             smiles = smiles or info.smiles
             InChI = InChI or info.InChI
             InChI_key = InChI_key or info.InChI_key
@@ -1142,10 +1143,9 @@ class Chemical:
         self._init_groups(InChI_key)
         if CAS == '56-81-5': # TODO: Make this part of data
             self._Dortmund = DortmundGroupCounts({2: 2, 3: 1, 14: 2, 81: 1})
-        self._init_data(CAS, MW, Tm, Tb, Tc, Pc,
-                        Vc, omega, Pt, Tt, Hfus, Hf, LHV,
-                        HHV, combustion, dipole, similarity_variable,
-                        iscyclic_aliphatic)
+        atoms = self.atoms
+        self._init_data(CAS, MW, Tm, Tb, Tc, Pc, Vc, omega, Pt, Tt, Hfus,
+                        dipole, atoms, similarity_variable, iscyclic_aliphatic)
         self._init_eos(eos, self._Tc, self._Pc, self._omega)
         self._init_handles(CAS, self._MW, self._Tm, self._Tb, self._Tc,
                            self._Pc, self.Zc, self._Vc,
@@ -1155,6 +1155,7 @@ class Chemical:
         self._init_energies(self._Cn, self._Hvap, self._Psat, self._Hfus,
                             self._Tm, self._Tb, self._eos, self._eos_1atm,
                             phase_ref)
+        self._init_reactions(Hf, LHV, HHV, combustion, atoms)
         if self._formula and self._Hf is not None: self.reset_combustion_data()
 
     def reset_combustion_data(self, method="Stoichiometry"):
@@ -1205,10 +1206,8 @@ class Chemical:
         else:
             self._PSRK = PSRKGroupCounts()
 
-    def _init_data(self, CAS, MW, Tm, Tb, Tc, Pc,
-                   Vc, omega, Pt, Tt, Hfus, Hf, LHV,
-                   HHV, combustion, dipole, similarity_variable,
-                   iscyclic_aliphatic):
+    def _init_data(self, CAS, MW, Tm, Tb, Tc, Pc, Vc, omega, Pt, Tt, Hfus,
+                   dipole, atoms, similarity_variable, iscyclic_aliphatic):
         self._MW = MW
         self._Tm = Tm or normal_melting_point_temperature(CAS)
         self._Tb = Tb = Tb or normal_boiling_point_temperature(CAS)
@@ -1224,27 +1223,28 @@ class Chemical:
         self._Tt = Tt or triple_point_temperature(CAS)
 
         # Energy
-        self._Hfus = Hfus if Hfus is not None else heat_of_fusion(CAS)
+        self._Hfus = heat_of_fusion(CAS) if Hfus is None else Hfus
+        
+        # Other
+        self._dipole = dipole or dipole_moment(CAS)
+        atoms = atoms or self.atoms
+        if atoms and not similarity_variable:
+            similarity_variable = compute_similarity_variable(atoms, MW)
+        self._similarity_variable = similarity_variable
+        self._iscyclic_aliphatic = iscyclic_aliphatic or False
 
-        # Chemistry
-        self._Hf = Hf if Hf is not None else heat_of_formation(CAS)
-        atoms = self.atoms
+    def _init_reactions(self, Hf, LHV, HHV, combustion, atoms):
+        self._Hf = heat_of_formation(self._CAS, self._phase_ref) if Hf is None else Hf
+        atoms = atoms or self.atoms
         if not all([LHV, HHV, combustion]) and atoms and Hf:
             CD = CombustionData.from_chemical_data(
-                 atoms, self._CAS, MW, self._Hf)
+                 atoms, self._CAS, self._MW, self._Hf)
             LHV = CD.LHV
             HHV = CD.HHV
             combustion = CD.stoichiometry
         self._LHV = LHV
         self._HHV = HHV
         self._combustion = combustion
-        
-        # Other
-        self._dipole = dipole or dipole_moment(CAS)
-        if atoms and not similarity_variable:
-            similarity_variable = compute_similarity_variable(atoms, MW)
-        self._similarity_variable = similarity_variable
-        self._iscyclic_aliphatic = iscyclic_aliphatic or False
 
     def _init_eos(self, eos, Tc, Pc, omega):
         self._eos = create_eos(eos, Tc, Pc, omega)
@@ -1690,12 +1690,11 @@ class Chemical:
     
     def show(self):
         """Print all specifications"""
-        getfield = getattr
         info = chemical_identity(self, pretty=True)
         for header, fields in _chemical_fields.items():
             section = []
             for field in fields:
-                value = getfield(self, field)
+                value = getattr(self, field)
                 field = field.lstrip('_')
                 if value is None:
                     line = f"{field}: None"
