@@ -128,7 +128,7 @@ class ChemicalIndexer(Indexer):
     """
     __slots__ = ('_chemicals', '_phase', '_data_cache')
     
-    def __new__(cls, phase, units=None, chemicals=None, **ID_data):
+    def __new__(cls, phase=NoPhase, units=None, chemicals=None, **ID_data):
         if ID_data:
             chemicals = tmo.settings.get_default_chemicals(chemicals)
             self = cls.from_data(chemicals.kwarray(ID_data), phase, chemicals)
@@ -188,7 +188,7 @@ class ChemicalIndexer(Indexer):
         return self
     
     @classmethod
-    def from_data(cls, data, phase, chemicals=None, check_data=True):
+    def from_data(cls, data, phase=NoPhase, chemicals=None, check_data=True):
         self = _new(cls)
         self._load_chemicals(chemicals)
         self._phase = Phase.convert(phase)
