@@ -60,6 +60,7 @@ References
    
 """
 import numpy as np
+import flexsolve as flx
 from scipy.interpolate import interp2d
 from ..base import (InterpolatedTDependentModel, 
                     ThermoModelHandle,
@@ -109,12 +110,12 @@ _VDI_PPDS_10 = read('VDI PPDS Thermal conductivity of gases.tsv')
 
 @kappa.l(ref='[1]_')
 def Sheffy_Johnson(T, MW, Tm):
-    r'''
+    r"""
     Notes
     -----
     Calculates the thermal conductivity of a liquid as a function of
     temperature using the Sheffy-Johnson (1961) method. Requires
-    Temperature, molecular weight, and melting point:
+    temperature, molecular weight, and melting point:
     
     .. math::
         k = 1.951 \frac{1-0.00126(T-T_m)}{T_m^{0.216}MW^{0.3}}
@@ -128,12 +129,12 @@ def Sheffy_Johnson(T, MW, Tm):
     >>> f(300)
     0.17740150413112196
     
-    '''
+    """
     return 1.951*(1 - 0.00126*(T - Tm))/(Tm**0.216*MW**0.3)
 
 @kappa.l(ref='[4]_')
 def Sato_Riedel(T, MW, Tb, Tc):
-    r'''
+    r"""
     Notes
     -----
     Calculates the thermal conductivity of a liquid as a function of
@@ -154,14 +155,14 @@ def Sato_Riedel(T, MW, Tb, Tc):
     >>> f(300)
     0.21037692461337687
     
-    '''
+    """
     Tr = T/Tc
     Tbr = Tb/Tc
     return 1.1053*(3. + 20.*(1 - Tr)**(2/3.))*MW**-0.5/(3. + 20.*(1 - Tbr)**(2/3.))
 
 @kappa.l(ref='[5]_')
 def Lakshmi_Prasad(T, MW):
-    r'''
+    r"""
     Notes
     -----
     Estimates thermal conductivity of pure liquids as a function of
@@ -181,12 +182,20 @@ def Lakshmi_Prasad(T, MW):
     >>> f(273.15)
     0.013664450000000009
     
-    '''
+    """
     return 0.0655 - 0.0005*T + (1.3855 - 0.00197*T)*MW**-0.5
 
+def Tmax_Lakshmi_Prasad(MW):
+    """
+    Returns the maximum temperature at which the Lakshmi Prasad method is
+    valid.
+    """
+    T_max = flx.aitken_secant(Lakshmi_Prasad.function, 298.15, args=(MW,))
+    return T_max - 10 # As an extra precaution
+
 @kappa.l(ref='[6]_')
 def Gharagheizi_liquid(T, MW, Tb, Pc, omega):
-    r'''
+    r"""
     Notes
     -----
     Estimates the thermal conductivity of a liquid as a function of
@@ -213,7 +222,7 @@ def Gharagheizi_liquid(T, MW, Tb, Pc, omega):
     >>> f(300)
     0.2171113029534838
     
-    '''
+    """
     Pc = Pc/1E5
     B = 16.0407*MW + 2.*Tb - 27.9074
     A = 3.8588*MW**8*(1.0045*B + 6.5152*MW - 8.9756)
@@ -223,7 +232,7 @@ def Gharagheizi_liquid(T, MW, Tb, Pc, omega):
 
 @kappa.l(ref='[7]_')
 def Nicola(T, MW, Tc, Pc, omega):
-    r'''
+    r"""
     Notes
     -----
     Estimates the thermal conductivity of a liquid as a function of
@@ -244,14 +253,14 @@ def Nicola(T, MW, Tc, Pc, omega):
     >>> f(300)
     0.10863821554584034
     
-    '''
+    """
     Tr = T/Tc
     Pc = Pc/1E5
     return 0.5147*(-0.2537*Tr + 0.0017*Pc + 0.1501*omega + (1./MW)**0.2999)
 
 @kappa.l(ref='[8]_')
 def Bahadori_liquid(T, MW):
-    r'''
+    r"""
     Notes
     -----
     Estimates the thermal conductivity of parafin liquid hydrocarbons.
@@ -260,9 +269,13 @@ def Bahadori_liquid(T, MW):
     
     .. math::
         K = a + bY + CY^2 + dY^3
+        
         a = A_1 + B_1 X + C_1 X^2 + D_1 X^3
+        
         b = A_2 + B_2 X + C_2 X^2 + D_2 X^3
+        
         c = A_3 + B_3 X + C_3 X^2 + D_3 X^3
+        
         d = A_4 + B_4 X + C_4 X^2 + D_4 X^3
     
     The accuracy of this equation has not been reviewed.
@@ -274,7 +287,7 @@ def Bahadori_liquid(T, MW):
     >>> f(273.15)
     0.14274278108272603
     
-    '''
+    """
     A = [-6.48326E-2, 2.715015E-3, -1.08580E-5, 9.853917E-9]
     B = [1.565612E-2, -1.55833E-4, 5.051114E-7, -4.68030E-10]
     C = [-1.80304E-4, 1.758693E-6, -5.55224E-9, 5.201365E-12]
@@ -289,7 +302,7 @@ def Bahadori_liquid(T, MW):
 
 @kappa.l(ref='[9]_')
 def Mersmann_Kind_thermal_conductivity_liquid(T, MW, Tc, Vc, atoms):
-    r'''
+    r"""
     Notes
     -----
     Estimates the thermal conductivity of organic liquid substances
@@ -314,7 +327,7 @@ def Mersmann_Kind_thermal_conductivity_liquid(T, MW, Tc, Vc, atoms):
     >>> f(400)
     0.08952713798442789
     
-    '''
+    """
     na = sum(atoms.values())
     lambda_star = 2/3.*(na + 40.*(1. - T/Tc)**0.5)
     Vc = Vc*1000 # m^3/mol to m^3/kmol
@@ -326,7 +339,7 @@ def Mersmann_Kind_thermal_conductivity_liquid(T, MW, Tc, Vc, atoms):
 
 @kappa.l(ref='[11]_')
 def DIPPR9G(T, P, Tc, Pc, k_l):
-    r'''
+    r"""
     Notes
     -----
     Adjusts for pressure. the thermal conductivity of a liquid using an
@@ -349,7 +362,7 @@ def DIPPR9G(T, P, Tc, Pc, k_l):
     >>> f(515.05)
     0.0864419738671184
     
-    '''
+    """
     Tr = T/Tc
     Pr = P/Pc
     if isinstance(k_l, ThermoModelHandle): k_l = k_l.at_T(T)
@@ -366,7 +379,7 @@ Qfunc_Missenard = interp2d(Prs_Missenard, Trs_Missenard, Qs_Missenard)
 
 @kappa.l(ref='[12]_')
 def Missenard(T, P, Tc, Pc, k_l):
-    r'''
+    r"""
     Notes
     -----
     Adjusts for pressure. the thermal conductivity of a liquid using an
@@ -387,7 +400,7 @@ def Missenard(T, P, Tc, Pc, k_l):
     >>> f(304.)
     0.2198375777069657
     
-    '''
+    """
     Tr = T/Tc
     Pr = P/Pc
     Q = float(Qfunc_Missenard(Pr, Tr))
@@ -428,14 +441,15 @@ def thermal_conductivity_liquid_handle(handle, CAS, MW, Tm, Tb, Tc, Pc, omega):
         add_model(Sheffy_Johnson.from_args(data), 0, 793.65)
     data = (MW,)
     if MW:
-        add_model(Lakshmi_Prasad.from_args(data))
+        Tmax = Tmax_Lakshmi_Prasad(MW)
+        add_model(Lakshmi_Prasad.from_args(data), 0., Tmax)
         add_model(Bahadori_liquid.from_args(data))
 
 ### Thermal Conductivity of Gases
 
 @kappa.g(ref='[13]_')
 def Eucken(T, MW, Cn, mu_g):
-    r'''
+    r"""
     Notes
     -----
     Estimates the thermal conductivity of a gas as a function of
@@ -458,7 +472,7 @@ def Eucken(T, MW, Cn, mu_g):
     >>> f(373.15)
     0.018792644287722975
     
-    '''
+    """
     if callable(Cn): Cn = Cn(T)
     if callable(mu_g): mu_g = mu_g.at_T(T)
     Cv = Cn - R
@@ -467,7 +481,7 @@ def Eucken(T, MW, Cn, mu_g):
 
 @kappa.g(ref='[13]_')
 def Eucken_modified(T, MW, Cn, mu):
-    r'''
+    r"""
     Notes
     -----
     Estimates the thermal conductivity of a gas as a function of
@@ -488,7 +502,7 @@ def Eucken_modified(T, MW, Cn, mu):
     >>> f(373.15)
     0.023593536999201956
     
-    '''
+    """
     if callable(Cn): Cn = Cn(T)
     if callable(mu): mu = mu.at_T(T)
     Cv = Cn - R
@@ -519,7 +533,7 @@ def DIPPR9B_nonlinear(T, MW, Cn, mu):
 
 @kappa.g(ref='[14]_')
 def Chung(T, MW, Tc, omega, Cn, mu):
-    r'''
+    r"""
     Notes
     -----
     Estimates the thermal conductivity of a gas as a function of
@@ -547,7 +561,7 @@ def Chung(T, MW, Tc, omega, Cn, mu):
     >>> f(T=373.15)
     0.023015653729496946
     
-    '''
+    """
     if callable(Cn): Cn = Cn(T)
     if callable(mu): mu = mu.at_T(T)
     Cv = Cn - R 
@@ -561,7 +575,7 @@ def Chung(T, MW, Tc, omega, Cn, mu):
 
 @kappa.g(ref='[13]_ [15]_')
 def Eli_Hanley(T, MW, Tc, Vc, Zc, omega, Cn):
-    r'''
+    r"""
     Notes
     -----
     Estimates the thermal conductivity of a gas as a function of
@@ -606,7 +620,7 @@ def Eli_Hanley(T, MW, Tc, Vc, Zc, omega, Cn):
     >>> f(T=373.15)
     0.02247951789135337
     
-    '''
+    """
     Cs = [2.907741307E6, -3.312874033E6, 1.608101838E6, -4.331904871E5, 
           7.062481330E4, -7.116620750E3, 4.325174400E2, -1.445911210E1, 2.037119479E-1]
     if callable(Cn): Cn = Cn(T)
@@ -628,7 +642,7 @@ def Eli_Hanley(T, MW, Tc, Vc, Zc, omega, Cn):
 
 @kappa.g(ref='[6]_')
 def Gharagheizi_gas(T, MW, Tb, Pc, omega):
-    r'''
+    r"""
     Notes
     -----
     Estimates the thermal conductivity of a gas as a function of
@@ -656,7 +670,7 @@ def Gharagheizi_gas(T, MW, Tb, Pc, omega):
     >>> f(580.)
     0.09594861261873211
     
-    '''
+    """
     Pc = Pc/1E4
     B = T + (2.*omega + 2.*T - 2.*T*(2.*omega + 3.2825)/Tb + 3.2825)/(2*omega + T - T*(2*omega+3.2825)/Tb + 3.2825) - T*(2*omega+3.2825)/Tb
     A = (2*omega + T - T*(2*omega + 3.2825)/Tb + 3.2825)/(0.1*MW*Pc*T) * (3.9752*omega + 0.1*Pc + 1.9876*B + 6.5243)**2
@@ -664,7 +678,7 @@ def Gharagheizi_gas(T, MW, Tb, Pc, omega):
 
 @kappa.g(ref='[16]_')
 def Bahadori_gas(T, MW):
-    r'''
+    r"""
     Notes
     -----
     Estimates the thermal conductivity of hydrocarbons gases at low P.
@@ -689,7 +703,7 @@ def Bahadori_gas(T, MW):
     >>> f(313.15)
     0.031968165337873326
     
-    '''
+    """
     A = [4.3931323468E-1, -3.88001122207E-2, 9.28616040136E-4, -6.57828995724E-6]
     B = [-2.9624238519E-3, 2.67956145820E-4, -6.40171884139E-6, 4.48579040207E-8]
     C = [7.54249790107E-6, -6.46636219509E-7, 1.5124510261E-8, -1.0376480449E-10]
@@ -706,7 +720,7 @@ def Bahadori_gas(T, MW):
 
 @kappa.g(ref='[17]_')
 def Stiel_Thodos_dense(T,P, MW, Tc, Pc, Vc, Zc, V_g, k_g):
-    r'''
+    r"""
     Notes
     -----
     Estimates the thermal conductivity of a gas at high pressure as a
@@ -738,7 +752,7 @@ def Stiel_Thodos_dense(T,P, MW, Tc, Pc, Vc, Zc, V_g, k_g):
     >>> f(T=378.15)
     0.041245574404863684
     
-    '''
+    """
     if callable(V_g): V_g = V_g(T, P)
     if isinstance(k_g, ThermoModelHandle): k_g = k_g(T, P)
     gamma = 210*(Tc*MW**3./(Pc/1E5)**4)**(1/6.)
@@ -756,7 +770,7 @@ def Stiel_Thodos_dense(T,P, MW, Tc, Pc, Vc, Zc, V_g, k_g):
 
 @kappa.g(ref='[13]_ [15]_')
 def Eli_Hanley_dense(T, P, MW, Tc, Vc, Zc, omega, Cn, V_g):
-    r'''
+    r"""
     Notes
     -----
     Estimates the thermal conductivity of a gas at high pressure as a
@@ -814,7 +828,7 @@ def Eli_Hanley_dense(T, P, MW, Tc, Vc, Zc, omega, Cn, V_g):
     >>> f(T=473.)
     0.06038475936515042
     
-    '''
+    """
     Cs = [2.907741307E6, -3.312874033E6, 1.608101838E6, -4.331904871E5,
           7.062481330E4, -7.116620750E3, 4.325174400E2, -1.445911210E1,
           2.037119479E-1]
@@ -875,7 +889,7 @@ def Eli_Hanley_dense(T, P, MW, Tc, Vc, Zc, omega, Cn, V_g):
 
 @kappa.g(other_params=['association'])
 def Chung_dense(T, P, MW, Tc, Vc, omega, Cn, V_g, mu_g, dipole, association=0):
-    r'''
+    r"""
     Other Parameters
     ----------------
     association : float, optional
@@ -931,7 +945,7 @@ def Chung_dense(T, P, MW, Tc, Vc, omega, Cn, V_g, mu_g, dipole, association=0):
     >>> f(T=473.)
     0.06160570379787278
     
-    '''
+    """
     if callable(V_g): V_g = V_g(T, P)
     if callable(mu_g): mu_g = mu_g(T, P)
     if callable(Cn): Cn = Cn(T)
